varying vec3 normal, pos, eyepos, material;
varying vec4 rgb;

float smoothen(float x)
{
  return x * x;
}

bool is_solid()
{
  //float prod1 = fract(dot(pos, vec3(2.0, 2.0, -2.0)));
  //float prod2 = fract(dot(pos, vec3(2.0, -2.0, 2.0)));
  //float prod3 = fract(dot(pos, vec3(-2.0, 2.0, 2.0)));

  float rand = 2.0 * (fract(sin(dot(gl_FragCoord.xy, vec2(12.9898, 78.233))) * 43758.5453) - 0.5);

  float prod1 = dot(pos, vec3(2.0, 2.0, -2.0));
  float diff1 = fwidth(prod1);
  if (fract(prod1 + rand * smoothen(diff1)) < 0.2) return true;

  float prod2 = dot(pos, vec3(2.0, -2.0, 2.0));
  float diff2 = fwidth(prod2);
  if (fract(prod2 + rand * smoothen(diff2)) < 0.2) return true;

  float prod3 = dot(pos, vec3(-2.0, 2.0, 2.0));
  float diff3 = fwidth(prod3);
  return fract(prod3 + rand * smoothen(diff3)) < 0.2;
}

void main()
{
  float surf = material.r;

  if (surf > 0.25 && (surf > 0.75 || is_solid()))
    {
      float lambert;
      float phong;
      float intensity;
      float highlight;

      vec3 l = normalize(vec3(gl_LightSource[0].position) - eyepos);
      vec3 e = normalize(-eyepos);
      vec3 n = normalize(normal);
      vec3 r = normalize(reflect(-l, n));

      lambert = dot(l, n);
      phong = max(dot(r, e), 0.0);

      if (gl_FrontFacing)
	{
	  lambert = -lambert;
	}

      intensity = 0.3 + max(0.0, lambert) * 0.5 + pow(phong, 5.0) * 0.3;
      highlight = pow(phong, material.b) * min(1.0, material.b * 0.04);

      gl_FragColor = intensity * rgb + highlight * vec4(1.0, 1.0, 1.0, 0.0);
      gl_FragColor.a = 1.0;

      gl_FragDepth = gl_FragCoord.z * material.g;
    }
  else
    {
      gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
      gl_FragDepth = gl_DepthRange.far;
    }
}
